@@ -2,7 +2,7 @@
 
 import gc
 import sys
-from typing import Any
+from typing import Callable
 from unittest import mock
 
 import pytest
@@ -16,6 +16,17 @@ from aiohttp.helpers import TimerNoop
 from aiohttp.test_utils import make_mocked_coro
 
 
+class WriterMock(mock.AsyncMock):
+    def __await__(self) -> None:
+        return self().__await__()
+
+    def add_done_callback(self, cb: Callable[[], None]) -> None:
+        cb()
+
+    def done(self) -> bool:
+        return True
+
+
 @pytest.fixture
 def session():
     return mock.Mock()
@@ -28,7 +39,7 @@ async def test_http_processing_error(session) -> None:
         "get",
         URL("http://del-cl-resp.org"),
         request_info=request_info,
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -56,7 +67,7 @@ def test_del(session) -> None:
         "get",
         URL("http://del-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -83,7 +94,7 @@ def test_close(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -104,7 +115,7 @@ def test_wait_for_100_1(loop, session) -> None:
         URL("http://python.org"),
         continue100=object(),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         timer=TimerNoop(),
         traces=[],
         loop=loop,
@@ -120,7 +131,7 @@ def test_wait_for_100_2(loop, session) -> None:
         URL("http://python.org"),
         request_info=mock.Mock(),
         continue100=None,
-        writer=mock.Mock(),
+        writer=WriterMock(),
         timer=TimerNoop(),
         traces=[],
         loop=loop,
@@ -135,7 +146,7 @@ def test_repr(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -152,7 +163,7 @@ def test_repr_non_ascii_url() -> None:
         "get",
         URL("http://fake-host.org/\u03bb"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -167,7 +178,7 @@ def test_repr_non_ascii_reason() -> None:
         "get",
         URL("http://fake-host.org/path"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -201,7 +212,7 @@ async def test_read_and_release_connection(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -227,7 +238,7 @@ async def test_read_and_release_connection_with_error(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -248,7 +259,7 @@ async def test_release(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -277,7 +288,7 @@ async def test_release_on_del(loop, session) -> None:
             "get",
             URL("http://def-cl-resp.org"),
             request_info=mock.Mock(),
-            writer=mock.Mock(),
+            writer=WriterMock(),
             continue100=None,
             timer=TimerNoop(),
             traces=[],
@@ -297,7 +308,7 @@ async def test_response_eof(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=None,
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -318,7 +329,7 @@ async def test_response_eof_upgraded(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -339,7 +350,7 @@ async def test_response_eof_after_connection_detach(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=None,
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -360,7 +371,7 @@ async def test_text(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -387,7 +398,7 @@ async def test_text_bad_encoding(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -417,7 +428,7 @@ async def test_text_custom_encoding(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -442,15 +453,13 @@ async def test_text_custom_encoding(loop, session) -> None:
 
 
 @pytest.mark.parametrize("content_type", ("text/plain", "text/plain;charset=invalid"))
-async def test_text_charset_resolver(
-    content_type: str, loop: Any, session: Any
-) -> None:
+async def test_text_charset_resolver(content_type: str, loop, session) -> None:
     session._resolve_charset = lambda r, b: "cp1251"
     response = ClientResponse(
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -479,7 +488,7 @@ async def test_get_encoding_body_none(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -509,7 +518,7 @@ async def test_text_after_read(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -536,7 +545,7 @@ async def test_json(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -563,7 +572,7 @@ async def test_json_extended_content_type(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -592,7 +601,7 @@ async def test_json_custom_content_type(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -619,7 +628,7 @@ async def test_json_custom_loader(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -641,7 +650,7 @@ async def test_json_invalid_content_type(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -650,11 +659,13 @@ async def test_json_invalid_content_type(loop, session) -> None:
     )
     response._headers = {"Content-Type": "data/octet-stream"}
     response._body = b""
+    response.status = 500
 
     with pytest.raises(aiohttp.ContentTypeError) as info:
         await response.json()
 
     assert info.value.request_info == response.request_info
+    assert info.value.status == 500
 
 
 async def test_json_no_content(loop, session) -> None:
@@ -662,7 +673,7 @@ async def test_json_no_content(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -681,7 +692,7 @@ async def test_json_override_encoding(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -710,7 +721,7 @@ def test_get_encoding_unknown(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -727,7 +738,7 @@ def test_raise_for_status_2xx() -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -744,7 +755,7 @@ def test_raise_for_status_4xx() -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -765,7 +776,7 @@ def test_raise_for_status_4xx_without_reason() -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -786,7 +797,7 @@ def test_resp_host() -> None:
         "get",
         URL("http://del-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -801,7 +812,7 @@ def test_content_type() -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -818,7 +829,7 @@ def test_content_type_no_header() -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -835,7 +846,7 @@ def test_charset() -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -852,7 +863,7 @@ def test_charset_no_header() -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -869,7 +880,7 @@ def test_charset_no_charset() -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -886,7 +897,7 @@ def test_content_disposition_full() -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -909,7 +920,7 @@ def test_content_disposition_no_parameters() -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -928,7 +939,7 @@ def test_content_disposition_no_header() -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -947,7 +958,7 @@ def test_response_request_info() -> None:
         "get",
         URL(url),
         request_info=RequestInfo(url, "get", headers),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -966,7 +977,7 @@ def test_request_info_in_exception() -> None:
         "get",
         URL(url),
         request_info=RequestInfo(url, "get", headers),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -987,7 +998,7 @@ def test_no_redirect_history_in_exception() -> None:
         "get",
         URL(url),
         request_info=RequestInfo(url, "get", headers),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -1010,7 +1021,7 @@ def test_redirect_history_in_exception() -> None:
         "get",
         URL(url),
         request_info=RequestInfo(url, "get", headers),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -1024,7 +1035,7 @@ def test_redirect_history_in_exception() -> None:
         "get",
         URL(hist_url),
         request_info=RequestInfo(url, "get", headers),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -1053,7 +1064,7 @@ async def test_response_read_triggers_callback(loop, session) -> None:
         response_method,
         response_url,
         request_info=mock.Mock,
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         loop=loop,
@@ -1086,7 +1097,7 @@ def test_response_real_url(loop, session) -> None:
         "get",
         url,
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -1103,7 +1114,7 @@ def test_response_links_comma_separated(loop, session) -> None:
         "get",
         url,
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -1133,7 +1144,7 @@ def test_response_links_multiple_headers(loop, session) -> None:
         "get",
         url,
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -1158,7 +1169,7 @@ def test_response_links_no_rel(loop, session) -> None:
         "get",
         url,
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -1177,7 +1188,7 @@ def test_response_links_quoted(loop, session) -> None:
         "get",
         url,
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -1200,7 +1211,7 @@ def test_response_links_relative(loop, session) -> None:
         "get",
         url,
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -1223,7 +1234,7 @@ def test_response_links_empty(loop, session) -> None:
         "get",
         url,
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -1239,7 +1250,7 @@ def test_response_not_closed_after_get_ok(mocker) -> None:
         "get",
         URL("http://del-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
